<?php
require_once('utils/saml2AC.php');
require_once('utils/sirAC.php');

class oauthAS {
    private $error;
    private $response;
    private $assertionChecking;
    protected $lifetime;
    protected $keys;
    private $isBearer;
    protected $activeDebug;

    function __construct() {
        $this->error=null;
        $this->response=null;
        $this->assertionChecking=null;
        $this->lifetime = 3600;
        $this->isBearer = false;
        $this->getKeys();
        $this->deactiveDebug();
    }

    /**
     * Function that manages the request of the app client and return an appropiate response.
     * Return an HTTP 400 Bad Request with
     *  a json_encode error in the body of the response
     * @param Array $post  Params of the request.   It contains:
     *        type REQUIRED. The parameter value MUST be set to "assertion".
     *        assertion_type REQUIRED. The type of the assertion as defined by the authorization server.
     *                                        The value MUST be an absolute URI. The types supported by this
     *                                        library are PAPI assertion and SAML2 assertion.
     *        assertion REQUIRED. The assertion.
     *        scope OPTIONAL. The scope of the access request expressed as a list of space-delimited strings.
     *                                        The value of the "scope" parameter is defined by the authorization server.
     *                                        If the value contains multiple space-delimited strings, their order does not
     *                                        matter, and each string adds an additional access range to the requested
     *                                         scope.
      *        lifetime OPTIONAL. The lifetime of the access token in seconds.
     * @param <type> $hostname IP of the Client requesting the access token.
     */
    public function manageRequest($post, $hostname) {
        if($this->activeDebug)error_log("oauthAS.manageRequest");
        if(array_key_exists("lifetime",$post) && $post['lifetime']!=null && $post['lifetime']!="") {
            $this->lifetime = new oauthAS($post['lifetime']);
        }
        $post['assertion'] = $this->transformArray(json_decode(urldecode($post['assertion'])));
        if($this->isValidRequest($post)) {
            $this->manageAccessToken($post, $hostname);
        }else {
            $this->error = "invalid_assertion";
        }
        if($this->error==null) {
            $this->generateJSONResponse($this->response);

        }else {
            header("HTTP/1.0 400 Bad Request");
            header("Content-Type: application/json");
            header("Cache-control:no-store");
            $err =  array("error" => $this->error);
            echo json_encode($err);
        }
    }

    private function transformArray($array) {
        $aux = array();
        foreach ($array as $key => $value) {
            if(is_array($value)) {
                $key = str_replace("_dos_puntos_",":", $key);
                $aux[$key] = $this->transformArray($value);
            }else {
                $key = str_replace("_dos_puntos_",":", $key);
                $value =  str_replace("_dos_puntos_",":", $value);
                $aux[$key] =$value;
            }
        }
        return $aux;
    }


    /**
     * Generates an appropiate "application/json"  response as defined by [RFC4627].
     * The parameters are serializedinto a JSON structure by adding each parameter at the highest
     *  structure level. Parameter names and string values are included as JSON strings. Numerical
     * values are included as JSON numbers.
     * For example: { "access_token":"SlAV32hkKG", "expires_in":3600, "refresh_token":"8xLOxBtZp8"}
     * @param array $response Params of the response:
     *      - access_token: REQUIRED
     *      - expires_in: OPTIONAL - The duration in seconds of the access token lifetime.
     *      - access_token_secret: REQUIRED if request by the client.
     */

    private function generateJSONResponse($response) {

        header("HTTP/1.0 200 Ok");
        header("Content-Type: application/json");
        header("Cache-control:no-store");
        echo json_encode($response);
    }
    /**
     * Function that generates an access token with the given parameters if them are valid parameters.
     * @param <type> $request Data request
     * @param <type> $hostname IP of the app client
     */
    private function manageAccessToken($request, $hostname) {
        if($this->activeDebug)error_log("oauthAS.manageAccessToken");
        $assertion= $request['assertion'];     
        if($this->assertionChecking->checkAssertion($assertion)) {
            $this->scope=$request['scope'];
            $this->response =  $this->generateAccessToken(
                    $this->assertionChecking->getPersonId(),
                    $this->scope,
                    $hostname
            );
        }else {
            $this->error = "invalid_user_credentials";
        }
    }

    /**
     * Function that generates an access token from the parameters.
     * @param String $personid Id of the user who makes the request
     * @param String $url_rs url of the service provider
     * @param String $scope URI of the user's request.
     * @param String $hostname IP of the client.
     * @return Array(wrap_access_token=>, wrap_access_token_expires_in=>)
     */
    private function generateAccessToken($personid, $scope, $hostname) {
        if($this->activeDebug) error_log("oauthAS.generateAccessToken");
        $array = array();
        if(!array_key_exists($hostname, $this->keys)) {
            $this->error = "Unregistered client app";
        }else {
            $key = $this->keys[$hostname];
            $time = microtime(true)+$this->lifetime;
            //hash_mac(M,secret)+M
            //$m = base64encode(spuc):base64encode(scope):base64encode(timestamp)
            //Token => hash_hmac(sha256,$m,$key):$m
            $message = base64_encode($personid).":".base64_encode($scope).":".base64_encode($time);
            $token= hash_hmac("sha256",$message,$key).":".$message;
            $digest = hash_hmac("sha256",$token,$hostname).":".$token;
            $array['access_token'] = $digest;
            $array['expires_in']=$this->lifetime;
            $array['scope']=$scope;
        }
        return $array;
    }


    /**
     * Function that checks if the request is a valid one.
     * @param <type> $param
     *      [type] => assertion
     *      [format] =>  Assertion format
     *      [assertion] => Assertion
     *      [scope] => uri of the scope
     * @return bool True if it's a valid request
     */
    private function isValidRequest($search) {
        if($this->activeDebug)error_log("oauthAS.isValidRequest ");
        $res=false;
        if(is_array($search)
                && array_key_exists("grant_type", $search)
                && (strcmp($search["grant_type"], "assertion")==0)
                && array_key_exists("assertion_type", $search)
                && array_key_exists("assertion", $search)
                && array_key_exists("scope", $search)
        ) {
            if($this->isValidAssertion($search["assertion_type"], $search['assertion'])) {
                $res=true;
            }else {
                $this->error="invalid_assertion";
            }
        }else {
            $this->error="unknown_format";
        }
        return $res;
    }

    /**
     * Checks if the assertion format is valid, depending of the assertion type (SAML2, PAPI)
     * @param Strinh $format
     * @param array $assertion
     * @return bool True if the assertion format it's  valid
     */
    private function isValidAssertion($assertion_type, $assertion) {
        if($this->activeDebug)error_log("oauthAS.isValidAssertion.urn");
        $dev = false;
        if(strcmp("urn:saml2",$assertion_type)==0) {
            $this->assertionChecking = new saml2AssertionChecking();
            if(!$this->assertionChecking->getError()) {
                $dev=true;
            }
        }else if (strcmp("urn:sir", $assertion_type)==0) {
            $this->assertionChecking = new sirAssertionChecking();
            if(!$this->assertionChecking->getError()) {
                $dev=true;
            }
        }
        return $dev;
    }

    /**
     * Function that get the keys of the clients, checking if the app client it's registered or not.
     * @param <type> $keysfile Keys archive
     */
    private function getKeys($keysfile="keys/keys.xml") {
        if($this->activeDebug) error_log("oauthAS.getKeys");
        $xml = simplexml_load_file($keysfile);
        if(strcmp($xml->getName(),"Keys")==0) {
            $arr = $xml->children();
            $aux =array();
            foreach($arr as $child) {
                $id=$child['id'];
                $val = $child['value'];
                $aux[(string)$id] = (string)$val;
            }
        }else {
            if($this->activeDebug) error_log("Bad formatting - keys.xml");
            $this->error = "Bad format of keys.xml";
        }
        $this->keys = $aux;
    }


    /**
     * Function that activates the error_log messages
     */
    private function activeDebug() {
        $this->activeDebug=true;
    }

    /**
     * Function that deactivates the error_log messages
     */
    private function deactiveDebug() {
        $this->activeDebug=false;
    }


    //Getters & Setters
    public function setLifetime($time) {
        if($this->activeDebug)error_log("oauthAS.setLifetime");
        $this->lifetime = $time;
    }
    public function getLifetime() {
        if($this->activeDebug)error_log("oauthAS.getLifetime");
        return $this->lifetime;
    }
}
?>